import logging
from lmp import (
    Client,
    TaskQueue,
    TaskProcessor,
    QueueConfig,
    Content,
    ContentType,
    PostAsyncInferRequest
)

# 配置日志
logging.basicConfig(level=logging.INFO)


def main():
    # 1. 创建任务队列
    config = QueueConfig(
        on_task_completed=lambda response: print(f"Task completed: {response.data.task_id if response.data else 'Unknown'}")
    )
    task_queue = TaskQueue(config)

    # 2. 创建客户端
    client = Client(
        token="ak-infer-bHBhaS1sbXA6bnhtamZoOm54bWpmaC1kZWZhdWx0OnpoYW5neGlucmFuM0BsaXhpYW5nLmNvbTppbmZlcg_NzMzNTcxNTktNmVlMC00MjU1LThjYzAtZmFmMDllZTRlNGM1",
        task_queue=task_queue,
        timeout=3600
    )

    # 3. 创建任务处理器
    processor = TaskProcessor(
        queue=task_queue,
        client=client,
        worker_num=100
    )

    # 4. 启动处理器
    processor.start()

    try:
        # 5. 单个请求
        request = PostAsyncInferRequest(
            contents=[
                Content(type=ContentType.TEXT, text="Hello, World!"),
                Content(
                    type=ContentType.IMAGE_URL,
                    image_url={"url": "datasets/ai-pictures/versions/0.1.0/0701-0707/1751332643437.png"}
                )
            ],
            model="qwen__qwen2_5-vl-72b-instruct" # 模型
        )

        response = client.post_async_infer(request)
        print(f"Task submitted: {response.data.task_id if response.data else 'Unknown'}")

        # 6. 批量请求
        batch_requests = [
            PostAsyncInferRequest(
                contents=[Content(type=ContentType.TEXT, text=f"Test {i}")],
                model="qwen__qwen2_5-vl-72b-instruct" # 模型
            )
            for i in range(200)
        ]

        batch_responses = client.post_async_infer_batch(batch_requests)
        print(f"Batch submitted: {len(batch_responses)} tasks")

        # 让程序运行一段时间
        import time
        time.sleep(3600)

    finally:
        # 7. 清理
        processor.stop()
        #task_queue.shutdown()
        client.close()


if __name__ == "__main__":
    main()


import logging
import threading
import time
from lmp import (
    Client,
    TaskQueue,
    TaskProcessor,
    QueueConfig,
    Content,
    ContentType,
    PostAsyncInferRequest
)

# 配置日志
logging.basicConfig(level=logging.INFO)


def main():
    # 1. 创建任务队列
    config = QueueConfig(
        on_task_completed=lambda response: print(f"Task completed: {response.data.task_id if response.data else 'Unknown'}")
    )
    task_queue = TaskQueue(config)

    # 2. 创建客户端
    client = Client(
        token="ak-infer-bHBhaS1sbXA6bnhtamZoOm54bWpmaC1kZWZhdWx0OnpoYW5neGlucmFuM0BsaXhpYW5nLmNvbTppbmZlcg_NzMzNTcxNTktNmVlMC00MjU1LThjYzAtZmFmMDllZTRlNGM1",
        task_queue=task_queue
    )

    # 3. 创建任务处理器
    processor = TaskProcessor(
        queue=task_queue,
        client=client,
        worker_num=5,
    )


    # 4. 启动处理器
    processor.start()
    threads = []

    # 创建并启动5个线程
    for worker_id in range(5):
        thread = threading.Thread(
            target=processor._worker,
            args=(worker_id,),
            name=f"Worker-{worker_id}",
            daemon=True
        )
        thread.start()
        threads.append(thread)
        print(f"Started Worker-{worker_id}")

    time.sleep(10)
"""
  try:
      # 5. 发送请求
      request = PostAsyncInferRequest(
          contents=[
              Content(type=ContentType.TEXT, text="Hello, World!"),
              Content(
                  type=ContentType.IMAGE_URL,
                  image_url={"url": "datasets/ad-vlm/versions/0.1.1/data/front.jpg"}
              )
          ],
          model="qwen__qwen2_5-vl-72b-instruct"
      )

      response = client.post_async_infer(request)
      print(f"Task submitted: {response.data.task_id if response.data else 'Unknown'}")

      # 6. 批量请求
      batch_requests = [
          PostAsyncInferRequest(
              contents=[Content(type=ContentType.TEXT, text=f"Test {i}")]
          )
          for i in range(5)
      ]

      batch_responses = client.post_async_infer_batch(batch_requests)
      print(f"Batch submitted: {len(batch_responses)} tasks")

      # 让程序运行一段时间
      import time
      time.sleep(60)

  finally:
      # 7. 清理
      processor.stop()
      # task_queue.shutdown()
      client.close()
"""

if __name__ == "__main__":
    main()


from .core import AwesomeWeatherClient
from .client import Client
from .task_queue import TaskQueue, QueueConfig
from .task_processor import TaskProcessor
from .models import (
    Content,
    ContentType,
    Message,
    PostAsyncInferRequest,
    PostAsyncInferResponse,
    TaskResponse,
    TaskStatus,
    Task
)
from .constants import DEFAULT_API_ENDPOINT, DEFAULT_MODEL
from .exceptions import LMPException, TaskTimeoutError, QueueFullError

__version__ = "1.0.0"
__author__ = "LMP SDK Team"

__all__ = [
    'AwesomeWeatherClient'
    "Client",
    "TaskQueue",
    "TaskProcessor",
    "QueueConfig",
    "Content",
    "ContentType",
    "Message",
    "PostAsyncInferRequest",
    "PostAsyncInferResponse",
    "TaskResponse",
    "TaskStatus",
    "Task",
    "DEFAULT_API_ENDPOINT",
    "DEFAULT_MODEL",
    "LMPException",
    "TaskTimeoutError",
    "QueueFullError",
]


import threading
import time
import logging
from typing import Optional

from .client import Client
from .task_queue import TaskQueue
from .constants import DEFAULT_WORKER_NUM

logger = logging.getLogger(__name__)


class TaskProcessor:
    """任务处理器"""

    def __init__(
        self,
        queue: TaskQueue,
        client: Client,
        worker_num: int = DEFAULT_WORKER_NUM
    ):
        """
        初始化任务处理器

        Args:
            queue: 任务队列
            client: 客户端
            worker_num: 工作线程数
        """
        self.queue = queue
        self.client = client
        self.worker_num = worker_num
        self.stop_event = threading.Event()
        self.workers = []

        logger.info(f"TaskProcessor initialized with {worker_num} workers")

    def start(self):
        """启动处理器"""
        logger.info(f"Starting {self.worker_num} workers...")

        for i in range(self.worker_num):
            worker = threading.Thread(
                target=self._worker,
                args=(i,),
                daemon=True
            )
            worker.start()
            self.workers.append(worker)

        logger.info("All workers started")

    def _worker(self, worker_id: int):
        """工作线程"""
        logger.info(f"Worker-{worker_id} started")

        while not self.stop_event.is_set():
            task = self.queue.get_next_task()

            if task is None:
                time.sleep(1)
                continue

            logger.info(f"Worker-{worker_id} processing task: {task.id}")

            try:
                self.client.wait_for_task_completion(
                    task.id,
                    self.queue.on_task_completed
                )
            except Exception as e:
                logger.error(f"Worker-{worker_id} failed to process task {task.id}: {e}")

        logger.info(f"Worker-{worker_id} stopped")

    def stop(self, timeout: float = 30):
        """停止处理器"""
        logger.info("Stopping TaskProcessor...")
        self.stop_event.set()

        for worker in self.workers:
            worker.join(timeout=timeout/len(self.workers))

        logger.info("TaskProcessor stopped")


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="lmp-sdk",
    version="1.0.0",
    author="LMP SDK Team",
    description="理想汽车 AI 平台 Python SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lixiang/lmp-sdk-python",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=[
        "requests>=2.25.0",
        "dataclasses>=0.6;python_version<'3.7'",
    ],
    extras_require={
        "dev": [
            "pytest>=6.0",
            "pytest-cov>=2.0",
            "black>=21.0",
            "flake8>=3.9",
            "mypy>=0.900",
        ]
    }
)


# API 端点
DEFAULT_API_ENDPOINT = "https://lpai.lixiang.com/lpai/api/lpai-service-lmp/async_infer"
BASE_GET_URL = "https://lpai.lixiang.com/lpai/api/lpai-service-lmp"

# 默认模型
DEFAULT_MODEL = "qwen__qwen2_5-vl-72b-instruct"

# 默认参数
DEFAULT_TEMPERATURE = 0.000001
DEFAULT_FREQUENCY_PENALTY = 1.05
DEFAULT_MAX_RETRIES = 5
DEFAULT_POLLING_INTERVAL = 10  # 秒
DEFAULT_MAX_WAIT_TIME = 86400  # 24小时（秒）
DEFAULT_MAX_QUEUE_SIZE = 10000
DEFAULT_WORKER_NUM = 5


import time
import logging
import threading
from typing import Optional, Callable, List
from datetime import timedelta
import requests

from .models import (
    PostAsyncInferRequest,
    PostAsyncInferResponse,
    PostAsyncInferParams,
    Message,
    TaskResponse,
    Content
)
from .constants import (
    DEFAULT_API_ENDPOINT,
    DEFAULT_MODEL,
    BASE_GET_URL,
    DEFAULT_TEMPERATURE,
    DEFAULT_MAX_RETRIES,
    DEFAULT_POLLING_INTERVAL,
    DEFAULT_MAX_WAIT_TIME
)
from .exceptions import APIError, TaskTimeoutError, TaskFailedError

logger = logging.getLogger(__name__)


class Client:
    """LMP 客户端"""

    def __init__(
        self,
        token: str,
        task_queue: Optional['TaskQueue'] = None,
        endpoint: str = DEFAULT_API_ENDPOINT,
        polling_interval: int = DEFAULT_POLLING_INTERVAL,
        max_wait_time: int = DEFAULT_MAX_WAIT_TIME,
        timeout: int = 300
    ):
        """
        初始化客户端

        Args:
            token: API Token
            task_queue: 任务队列（可选）
            endpoint: API 端点
            polling_interval: 轮询间隔（秒）
            max_wait_time: 最大等待时间（秒）
            timeout: HTTP 请求超时（秒）
        """
        self.endpoint = endpoint
        self.token = token
        self.task_queue = task_queue
        self.polling_interval = polling_interval
        self.max_wait_time = max_wait_time

        # 创建 Session
        self.session = requests.Session()
        self.session.headers.update({
            "content-type": "application/json",
            "accept": "application/json",
            "Authorization": f"Bearer {token}"
        })
        self.timeout = timeout

        logger.info(f"Client initialized with endpoint: {endpoint}")

    def post_async_infer(self, request: PostAsyncInferRequest) -> PostAsyncInferResponse:
        # 设置默认值
        if not request.model:
            request.model = DEFAULT_MODEL
        if request.temperature == 0:
            request.temperature = DEFAULT_TEMPERATURE
        if request.max_retries == 0:
            request.max_retries = DEFAULT_MAX_RETRIES

        if not request.contents:
            raise ValueError("No contents provided")

        # 构建参数
        params = PostAsyncInferParams(
            model=request.model,
            messages=[
                Message(role="user", content=request.contents)
            ],
            temperature=request.temperature,
            frequency_penalty=1.05,
            stream=False,
            ipai_max_request_retries=request.max_retries
        )

        return self.async_infer_send(params)

    def async_infer_send(self, params: PostAsyncInferParams) -> PostAsyncInferResponse:

        try:
            response = self.session.post(
                self.endpoint,
                json=params.to_dict(),
                timeout=self.timeout
            )

            if response.status_code != 200:
                raise APIError(response.status_code, response.text)

            result = PostAsyncInferResponse.from_dict(response.json())

            # 添加到任务队列
            if self.task_queue and result.data:
                self.task_queue.add_task(result.data.task_id)
                logger.info(f"Task {result.data.task_id} added to queue")

            return result

        except requests.RequestException as e:
            logger.error(f"Request failed: {e}")
            raise APIError(0, str(e))

    def get_task_status(self, task_id: str) -> TaskResponse:

        url = f"{BASE_GET_URL}/async_infer/{task_id}"

        try:
            response = self.session.get(url, timeout=self.timeout)

            if response.status_code != 200:
                raise APIError(response.status_code, response.text)

            return TaskResponse.from_dict(response.json())

        except requests.RequestException as e:
            logger.error(f"Get task status failed: {e}")
            raise APIError(0, str(e))

    def wait_for_task_completion(
        self,
        task_id: str,
        callback: Optional[Callable[[TaskResponse], None]] = None
    ) -> TaskResponse:
        start_time = time.time()
        deadline = start_time + self.max_wait_time

        logger.info(f"Start polling task: {task_id}")
        logger.info(f"Polling interval: {self.polling_interval}s, Max wait: {self.max_wait_time}s")

        request_count = 0

        while True:
            # 检查超时
            if time.time() > deadline:
                raise TaskTimeoutError(f"Task timeout after {time.time() - start_time:.1f}s")

            request_count += 1
            current_time = time.strftime("%H:%M:%S")
            logger.info(f"[{current_time}] Request #{request_count}")

            try:
                result = self.get_task_status(task_id)

                # 检查错误码
                if result.errno != 0:
                    logger.info(f"Task processing: {result.msg}")
                    if result.data and result.data.estimated_scheduled_time > 0:
                        logger.info(f"Estimated wait: {result.data.estimated_scheduled_time}s")
                    time.sleep(self.polling_interval)
                    continue

                # 检查任务状态
                if result.data:
                    status = result.data.status

                    if status == "RUNNING":
                        logger.info("Task running...")
                    elif status == "SUCCEEDED":
                        logger.info("Task succeeded!")
                        self._update_task_status(task_id, callback, result)
                        return result
                    elif status in ["FAILED", "UNKNOWN"]:
                        logger.error(f"Task failed: {result.data.failed_reason}")
                        self._update_task_status(task_id, callback, result)
                        raise TaskFailedError(f"Task failed: {result.data.failed_reason}")
                    elif status == "PENDING":
                        logger.info("Task pending...")
                    else:
                        logger.warning(f"Unknown status: {status}")

            except (APIError, TaskFailedError):
                raise
            except Exception as e:
                logger.error(f"Request failed: {e}")

            time.sleep(self.polling_interval)

    def _update_task_status(
        self,
        task_id: str,
        callback: Optional[Callable[[TaskResponse], None]],
        response: TaskResponse
    ):
        """更新任务状态"""
        # 从队列删除
        if self.task_queue:
            self.task_queue.remove_task(task_id)

        # 执行回调
        if callback:
            threading.Thread(target=callback, args=(response,), daemon=True).start()

    def post_async_infer_batch(
        self,
        requests: List[PostAsyncInferRequest],
        max_workers: int = 10
    ) -> List[PostAsyncInferResponse]:

        from concurrent.futures import ThreadPoolExecutor, as_completed

        results = []

        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            futures = {
                executor.submit(self.post_async_infer, req): i
                for i, req in enumerate(requests)
            }

            for future in as_completed(futures):
                try:
                    result = future.result()
                    results.append(result)
                except Exception as e:
                    logger.error(f"Batch request failed: {e}")
                    results.append(None)

        return results

    def close(self):
        """关闭客户端"""
        self.session.close()
        logger.info("Client closed")

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()